using System.Reflection;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.Diagnostics;
using System.Windows.Forms;

namespace WinFormIsDirtyCS
{
  public class WinFormControlsDirty_V1
  {
    #region '"Private Variables"'
    private int mintInitialHash;
    private int mintEndHash;
    private Form mfrm;
    #endregion

    #region '"Constructors"'
    public WinFormControlsDirty_V1(Form frm)
    {
      mfrm = frm;
    }
    #endregion

    #region '"InitControls Method"'
    public void InitControls()
    {
      //  Set the Initial Hash Value
      //  This will be used to compare against the ending hash value
      //  If they are different, then the form is dirty.
      mintInitialHash = GetControls(mfrm.Controls).GetHashCode();
    }
    #endregion

    #region '"IsDirty Method"'
    public bool IsDirty()
    {
      //  Get the Ending Hash Value
      mintEndHash = GetControls(mfrm.Controls).GetHashCode();

      return (mintInitialHash != mintEndHash);
    }
    #endregion

    #region '"Get Methods"'
    private string GetControls(System.Windows.Forms.Control.ControlCollection ctls)
    {
      System.Text.StringBuilder sb = new System.Text.StringBuilder(1024);

      foreach (System.Windows.Forms.Control ctl in ctls)
      {
        Debug.WriteLine(ctl.Name + " (" + ctl.GetType().ToString() + ")");

        if (ctl.HasChildren)
        {
          //  Recurse into any "Grouping" controls
          sb.Append(GetControls(ctl.Controls));
        }
        switch (ctl.GetType().ToString())
        {
          case "System.Windows.Forms.TextBox":
            sb.Append(GetControlValue(ctl, "Text"));
            break;
          case "System.Windows.Forms.NumericUpDown":
            sb.Append(GetControlValue(ctl, "Value"));
            break;
          case "System.Windows.Forms.CheckBox":
            sb.Append(GetControlValue(ctl, "Checked"));
            break;
          case "System.Windows.Forms.ComboBox":
            if (GetControlValue(ctl, "DropDownStyle") == "DropDownList")
            {
              sb.Append(GetControlValue(ctl, "SelectedIndex"));
            }
            else
            {
              sb.Append(GetControlValue(ctl, "Text"));
            }
            break;
          case "System.Windows.Forms.MaskedTextBox":
            sb.Append(GetControlValue(ctl, "Text"));
            break;
          case "System.Windows.Forms.RadioButton":
            sb.Append(GetControlValue(ctl, "Checked"));
            break;
          case "System.Windows.Forms.MonthCalendar":
            sb.Append(GetControlValue(ctl, "SelectionStart"));
            sb.Append(GetControlValue(ctl, "SelectionEnd"));
            break;
          case "System.Windows.Forms.DateTimePicker":
            sb.Append(GetControlValue(ctl, "Value"));
            break;
          case "System.Windows.Forms.ListBox":
            if (GetControlValue(ctl, "SelectionMode") == "One")
            {
              sb.Append(GetControlValue(ctl, "SelectedIndex"));
            }
            else
            {
              sb.Append(GetMultiListValues(((ListBox)(ctl))));
            }
            break;
          case "System.Windows.Forms.CheckedListBox":
            sb.Append(GetMultiListValues(((ListBox)(ctl))));
            break;
        }
      }

      return sb.ToString();
    }

    private string GetMultiListValues(ListBox ctl)
    {
      System.Text.StringBuilder sb = new System.Text.StringBuilder(200);

      //  Build a concatenated list of selected indices
      foreach (int intLoop in ctl.SelectedIndices)
      {
        sb.Append(intLoop.ToString());
      }

      return sb.ToString();
    }

    private string GetControlValue(Control ctl, string PropertyName)
    {
      Type t = null;

      t = ctl.GetType();

      //  Use Reflection to get the property value
      return Convert.ToString(t.InvokeMember(PropertyName, BindingFlags.DeclaredOnly | BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.GetProperty, null, ctl, null));
    }
    #endregion
  }
} 