using System.Reflection;
using System.Windows.Forms;
using System;
using System.Diagnostics;

namespace WinFormIsDirtyCS
{
  public class WinFormBase : System.Windows.Forms.Form
  {
    #region '"Private Variables"'
    private int mintInitialHash;
    #endregion

    #region '"InitControls Method"'
    public void InitControls()
    {
      //  Set the Initial Hash Value
      //  This will be used to compare against the ending hash value
      //  If they are different, then the form is dirty.
      mintInitialHash = GetControls(this.Controls).GetHashCode();
    }
    #endregion

    #region '"IsDirty Method"'
    public bool IsDirty()
    {
      int intHash = 0;

      //  Get the Ending Hash Value
      intHash = GetControls(this.Controls).GetHashCode();

      return (mintInitialHash != intHash);
    }
    #endregion

    #region '"Get Methods"'
    private string GetControls(Control.ControlCollection ctls)
    {
      System.Text.StringBuilder sb = new System.Text.StringBuilder(1024);
      DirtyControl dc = null;

      foreach (System.Windows.Forms.Control ctl in ctls)
      {
        Debug.WriteLine(ctl.Name + " (" + ctl.GetType().ToString() + ")");

        if (ctl.HasChildren)
        {
          //  Recurse into any "Grouping" controls
          sb.Append(GetControls(ctl.Controls));
        }

        //  Get DirtyControl Definition
        dc = AppConfig.DirtyControlsList.GetByType(ctl.GetType().ToString());
        if (dc != null)
        {
          if (dc.IsMultiList & dc.PropertyToCheck == string.Empty)
          {
            sb.Append(GetMultiListValues(((ListBox)(ctl))));
          }
          else if (dc.IsMultiList & dc.PropertyToCheck != string.Empty)
          {
            if (GetControlValue(ctl, dc.PropertyToCheck) == dc.PropertyToCheckValue)
            {
              sb.Append(GetControlValue(ctl, dc.Property1));
            }
            else
            {
              sb.Append(GetMultiListValues(((ListBox)(ctl))));
            }
          }
          else if (dc.IsMultiList == false & dc.PropertyToCheck != string.Empty)
          {
            if (GetControlValue(ctl, dc.PropertyToCheck) == dc.PropertyToCheckValue)
            {
              sb.Append(GetControlValue(ctl, dc.Property1));
            }
            else
            {
              sb.Append(GetControlValue(ctl, dc.Property2));
            }
          }
          else if (dc.IsMultiList == false & dc.Property2 != string.Empty)
          {
            sb.Append(GetControlValue(ctl, dc.Property1));
            sb.Append(GetControlValue(ctl, dc.Property2));
          }
          else
          {
            sb.Append(GetControlValue(ctl, dc.Property1));
          }
        }
      }

      return sb.ToString();
    }


    private string GetMultiListValues(ListBox ctl)
    {
      System.Text.StringBuilder sb = new System.Text.StringBuilder(200);

      //  Build a concatenated list of selected indices
      foreach (int intLoop in ctl.SelectedIndices)
      {
        sb.Append(intLoop.ToString());
      }

      return sb.ToString();
    }


    private string GetControlValue(Control ctl, string PropertyName)
    {
      Type t = null;

      t = ctl.GetType();

      //  Use Reflection to get the property value
      return Convert.ToString(t.InvokeMember(PropertyName, BindingFlags.DeclaredOnly | BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.GetProperty, null, ctl, null));
    }
    #endregion
  }
} 