Imports System.Reflection

Public Class WinFormControlsDirty_V1
#Region "Private Variables"
  Private mintInitialHash As Integer
  Private mintEndHash As Integer
  Private mfrm As Form
#End Region

#Region "Constructors"
  Public Sub New(ByVal frm As Form)
    mfrm = frm
  End Sub
#End Region

#Region "InitControls Method"
  Public Sub InitControls()
    ' Set the Initial Hash Value
    ' This will be used to compare against the ending hash value
    ' If they are different, then the form is dirty.
    mintInitialHash = GetControls(mfrm.Controls).GetHashCode()
  End Sub
#End Region

#Region "IsDirty Method"
  Public Function IsDirty() As Boolean
    ' Get the Ending Hash Value
    mintEndHash = GetControls(mfrm.Controls).GetHashCode()

    Return (mintInitialHash <> mintEndHash)
  End Function
#End Region

#Region "Get Methods"
  Private Function GetControls(ByVal ctls As System.Windows.Forms.Control.ControlCollection) As String
    Dim sb As New System.Text.StringBuilder(1024)

    For Each ctl As Control In ctls
      Debug.WriteLine(ctl.Name & " (" & ctl.GetType().ToString() & ")")

      If ctl.HasChildren Then
        ' Recurse into any "Grouping" controls
        sb.Append(GetControls(ctl.Controls))
      End If
      Select Case ctl.GetType().ToString()
        Case "System.Windows.Forms.TextBox"
          sb.Append(GetControlValue(ctl, "Text"))
        Case "System.Windows.Forms.NumericUpDown"
          sb.Append(GetControlValue(ctl, "Value"))
        Case "System.Windows.Forms.CheckBox"
          sb.Append(GetControlValue(ctl, "Checked"))
        Case "System.Windows.Forms.ComboBox"
          If GetControlValue(ctl, "DropDownStyle") = "DropDownList" Then
            sb.Append(GetControlValue(ctl, "SelectedIndex"))
          Else
            sb.Append(GetControlValue(ctl, "Text"))
          End If
        Case "System.Windows.Forms.MaskedTextBox"
          sb.Append(GetControlValue(ctl, "Text"))
        Case "System.Windows.Forms.RadioButton"
          sb.Append(GetControlValue(ctl, "Checked"))
        Case "System.Windows.Forms.MonthCalendar"
          sb.Append(GetControlValue(ctl, "SelectionStart"))
          sb.Append(GetControlValue(ctl, "SelectionEnd"))
        Case "System.Windows.Forms.DateTimePicker"
          sb.Append(GetControlValue(ctl, "Value"))
        Case "System.Windows.Forms.ListBox"
          If GetControlValue(ctl, "SelectionMode") = "One" Then
            sb.Append(GetControlValue(ctl, "SelectedIndex"))
          Else
            sb.Append(GetMultiListValues(CType(ctl, ListBox)))
          End If
        Case "System.Windows.Forms.CheckedListBox"
          sb.Append(GetMultiListValues(CType(ctl, ListBox)))
      End Select
    Next

    Return sb.ToString()
  End Function

  Private Function GetMultiListValues(ByVal ctl As ListBox) As String
    Dim sb As New System.Text.StringBuilder(200)

    ' Build a concatenated list of selected indices
    For Each intLoop As Integer In ctl.SelectedIndices
      sb.Append(intLoop.ToString())
    Next

    Return sb.ToString()
  End Function

  Private Function GetControlValue(ByVal ctl As Control, ByVal PropertyName As String) As String
    Dim t As Type

    t = ctl.GetType()

    ' Use Reflection to get the property value
    Return Convert.ToString(t.InvokeMember(PropertyName, BindingFlags.DeclaredOnly Or BindingFlags.Public Or BindingFlags.NonPublic Or BindingFlags.Instance Or BindingFlags.GetProperty, Nothing, ctl, Nothing))
  End Function
#End Region
End Class
