Imports System.Reflection
Imports System.Windows.Forms

Public Class WinFormBase
  Inherits System.Windows.Forms.Form

#Region "Private Variables"
  Private mintInitialHash As Integer
#End Region

#Region "InitControls Method"
  Public Sub InitControls()
    ' Set the Initial Hash Value
    ' This will be used to compare against the ending hash value
    ' If they are different, then the form is dirty.
    mintInitialHash = GetControls(Me.Controls).GetHashCode()
  End Sub
#End Region

#Region "IsDirty Method"
  Public Function IsDirty() As Boolean
    Dim intHash As Integer

    ' Get the Ending Hash Value
    intHash = GetControls(Me.Controls).GetHashCode()

    Return (mintInitialHash <> intHash)
  End Function
#End Region

#Region "Get Methods"
  Private Function GetControls( _
   ByVal ctls As Control.ControlCollection) As String
    Dim sb As New System.Text.StringBuilder(1024)
    Dim dc As DirtyControl

    For Each ctl As Control In ctls
      Debug.WriteLine(ctl.Name & " (" & ctl.GetType().ToString() & ")")

      If ctl.HasChildren Then
        ' Recurse into any "Grouping" controls
        sb.Append(GetControls(ctl.Controls))
      End If

      ' Get DirtyControl Definition
      dc = AppConfig.DirtyControlsList.GetByType( _
       ctl.GetType().ToString())
      If dc IsNot Nothing Then
        If dc.IsMultiList And dc.PropertyToCheck = String.Empty Then
          sb.Append(GetMultiListValues(CType(ctl, ListBox)))
        ElseIf dc.IsMultiList And dc.PropertyToCheck <> String.Empty Then
          If GetControlValue(ctl, dc.PropertyToCheck) = _
           dc.PropertyToCheckValue Then
            sb.Append(GetControlValue(ctl, dc.Property1))
          Else
            sb.Append(GetMultiListValues(CType(ctl, ListBox)))
          End If
        ElseIf dc.IsMultiList = False And _
                  dc.PropertyToCheck <> String.Empty Then
          If GetControlValue(ctl, dc.PropertyToCheck) = _
           dc.PropertyToCheckValue Then
            sb.Append(GetControlValue(ctl, dc.Property1))
          Else
            sb.Append(GetControlValue(ctl, dc.Property2))
          End If
        ElseIf dc.IsMultiList = False And dc.Property2 <> String.Empty Then
          sb.Append(GetControlValue(ctl, dc.Property1))
          sb.Append(GetControlValue(ctl, dc.Property2))
        Else
          sb.Append(GetControlValue(ctl, dc.Property1))
        End If
      End If
    Next

    Return sb.ToString()
  End Function

  Private Function GetMultiListValues(ByVal ctl As ListBox) As String
    Dim sb As New System.Text.StringBuilder(200)

    ' Build a concatenated list of selected indices
    For Each intLoop As Integer In ctl.SelectedIndices
      sb.Append(intLoop.ToString())
    Next

    Return sb.ToString()
  End Function

  Private Function GetControlValue( _
   ByVal ctl As Control, _
   ByVal PropertyName As String) As String
    Dim t As Type

    t = ctl.GetType()

    ' Use Reflection to get the property value
    Return Convert.ToString(t.InvokeMember( _
     PropertyName, BindingFlags.DeclaredOnly Or _
     BindingFlags.Public Or BindingFlags.NonPublic Or _
     BindingFlags.Instance Or BindingFlags.GetProperty, _
     Nothing, ctl, Nothing))
  End Function
#End Region
End Class
